require 'uu/os/server/abstract_middleware'
require 'uu/os/security/session'
require 'uu/os/server/immediate_response'
require 'uu/os/application/application_error'
require 'uu/os/server/request'
require 'uu/os/ipc/ipc_error'
require 'uu/os/ipc/error_kind'
require 'uu/os/ipc/error_codes'

module UU
  module OS
    module Application
      class AuthenticationMiddleware < UU::OS::Server::AbstractMiddleware

        def initialize(app)
          super(app)
        end

        def call(env)
          request = UU::OS::Server::Request.new(env)
          begin
            do_authentication(request)
            @app.call(env)
          ensure
            UU::OS::Security::Session.logout(true)
          end
        end

        # @private
        def do_authentication(request) # :nodoc:

          if request.authorization_header.nil? || request.authorization_header.empty?
            ex =  UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::CLIENT, UU::OS::Application::ErrorCodes::NOT_AUTHENTICATED, "Missing authentication data.")
            if request.get?
              realm = request.host_with_port
              header = {www_authenticate: "Basic realm=#{realm}"}
              header[:content_type] = 'application/json'
              raise UU::OS::Server::ImmediateResponse.new(401, header, ex.to_json)
            else
              raise ex
            end
          else
            begin
                UU::OS::Security::Session.login(request.authorization_header)
            rescue UU::OS::REST::ClientException => e
              if request.get?
                ex =  UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::CLIENT, UU::OS::Application::ErrorCodes::NOT_AUTHENTICATED, "Authentication failed.")
                realm = request.host_with_port
                header = {www_authenticate: "Basic realm=#{realm}"}
                header[:content_type] = 'application/json'
                raise UU::OS::Server::ImmediateResponse.new(401, header, ex.to_json)
              else
                raise UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::CLIENT, UU::OS::Application::ErrorCodes::NOT_AUTHENTICATED, "Authentication failed.", e)
              end
            rescue Exception => e
              raise UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::Application::ErrorCodes::NOT_AUTHENTICATED, "Authentication failed.", e)
            end
          end
        end
      end
    end
  end
end