require 'uu/os/server/abstract_middleware'
require 'uu/os/server/immediate_response'
require 'uu/os/server/request'
require 'uu/os/server/request'
require 'uu/os/application/application_error'
require 'uu/os/application/error_codes'
require 'uu/os/ipc/ipc_error'
require 'uu/os/ipc/error_kind'
require 'uu/os/ipc/error_codes'
require 'uu/os/util/response_builder'
require 'uu/os/rest/remote_exception'
require 'uu/os/application/log_helper'


module UU
  module OS
    module Application
      class ErrorHandlingMiddleware < UU::OS::Server::AbstractMiddleware
        include UU::OS::Util::ResponseBuilder
        include UU::OS::Application::LogHelper

        def initialize(app)
          super(app)
        end

        def call(env)
          begin
            return @app.call(env)


          rescue Exception => e
            request = UU::OS::Server::Request.new(env)
            log_error(request.logger, e)
            if raise_errors
              raise e
            end
            if e.kind_of? UU::OS::Server::ImmediateResponse
              return e.response
            end
            error_declared = false
            (declared_errors || []).each do |error_class|
              if e.kind_of?(error_class)
                error_declared = true
                break;
              end
            end
            if error_declared
              # Declared error should be passed directly to client.
              status = (e.respond_to?(:kind) && e.kind == UU::OS::IPC::ErrorKind::CLIENT) ? 400 : 500
              return response(status, {content_type: 'application/json'}, e.to_json)
            else
              # Wrap undeclared error to common error.
              if UU::OS::Server::ServerContext.test_or_development_mode?
                common_error = UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::Application::ErrorCodes::UNEXPECTED_PROBLEM,
                                                                         'Unexpected server error', e)
              else
                # Do not publish internal problem when not in development mode.
                common_error = UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::Application::ErrorCodes::UNEXPECTED_PROBLEM,
                                                                         'Unexpected server error')
              end
              # Use same exception ID to be able to find exception in logs.
              common_error.instance_variable_set(:@id, e.id)
              return response(500, {content_type: 'application/json'}, common_error.to_json)
            end
          end
        end

        # @private
        def declared_errors
           [UU::OS::IPC::IPCError, UU::OS::REST::RemoteException]
        end

        # @private
        def raise_errors
           false
        end

      end
    end
  end
end
