require 'uu/os/server/abstract_middleware'
require 'uu/os/logger/mdc_provider'
require 'uu/os/server/request'
require 'uu/os/application/config'
require 'uu/os/application/logger'

module UU
  module OS
    module Application
      class LoggerMiddleware < UU::OS::Server::AbstractMiddleware

        def initialize(app)
          super(app)
        end

        def call(env)
          init_logger(env)
          begin
            return @app.call(env)
          ensure
            clear_logger_context(env)
          end
        end

        # @private
        def init_logger(env)
          request = UU::OS::Server::Request.new(env)
          # Prepare MDC context for logger.
          # We need to initialize this even before invocation of prepare_execution
          # in order to be able to log as much errors as possible into uuApplog.
          UU::OS::Logger::MDCProvider.put('uu-appURI', UU::OS::Application::Config[:app_uri]) unless UU::OS::Application::Config[:app_uri].nil?
          UU::OS::Logger::MDCProvider.put('uu-applogURI', UU::OS::Application::Config[:applog_uri]) unless UU::OS::Application::Config[:applog_uri].nil?
          UU::OS::Logger::MDCProvider.put('uu-ucURI', URI(request.path).path)
          UU::OS::Logger::MDCProvider.put('uu-execID', SecureRandom.uuid)

          # TODO Solve setting of executive and access role to MDC
          # Executive role cannot be obtained without authorization
          # Reading of access role means additional performance hit
          # UU::OS::Logger::MDCProvider.put('uu-erURI', nil)
          # UU::OS::Logger::MDCProvider.put('uu-acrURI', UU::OS::Security::Session.get_access_role(uu_uri))

          # use both variables 'rack.logger' is standard but sum framework (e.g. Padrino) overwrite this varible
          # and do not care if it is set.
          env['uu.logger'] = logger
          env['rack.logger'] = env['uu.logger']
        end

        # @private
        def clear_logger_context(env)
          UU::OS::Logger::MDCProvider.get_context.each_key do |key|
            UU::OS::Logger::MDCProvider.remove(key)
          end
        end

        private

        # @private
        def logger
          UU::OS::Application::Logger.new
        end

      end
    end
  end

end
